package de.hft.stuttgart.quality.adapter.types;

import javax.xml.namespace.QName;

import org.xmlobjects.annotation.XMLElement;
import org.xmlobjects.builder.ObjectBuildException;
import org.xmlobjects.builder.ObjectBuilder;
import org.xmlobjects.serializer.ObjectSerializeException;
import org.xmlobjects.serializer.ObjectSerializer;
import org.xmlobjects.stream.XMLReadException;
import org.xmlobjects.stream.XMLReader;
import org.xmlobjects.stream.XMLWriteException;
import org.xmlobjects.stream.XMLWriter;
import org.xmlobjects.xml.Attributes;
import org.xmlobjects.xml.Element;
import org.xmlobjects.xml.Namespaces;

import de.hft.stuttgart.quality.QualityADEModule;
import de.hft.stuttgart.quality.adapter.properties.PolygonIdListPropertyAdapter;
import de.hft.stuttgart.quality.model.properties.PolygonIdListProperty;
import de.hft.stuttgart.quality.model.types.ComponentList;

@XMLElement(name = "ComponentList", namespaceURI = QualityADEModule.NAMESPACE_URI)
public class ComponentListAdapter implements ObjectBuilder<ComponentList>, ObjectSerializer<ComponentList> {

	@Override
	public ComponentList createObject(QName name, Object parent) throws ObjectBuildException {
		return new ComponentList();
	}

	@Override
	public void buildChildObject(ComponentList object, QName name, Attributes attributes, XMLReader reader)
			throws ObjectBuildException, XMLReadException {
		if (!QualityADEModule.NAMESPACE_URI.equals(name.getNamespaceURI())) {
			return;
		}
		switch (name.getLocalPart()) {
		case "component" -> object.getComponents()
				.add(reader.getObjectUsingBuilder(PolygonIdListPropertyAdapter.class));
		default -> throw new IllegalStateException(
				"Cannot handle name " + name + " when building ComponentList element");
		}
	}
	
	@Override
	public Element createElement(ComponentList object, Namespaces namespaces) throws ObjectSerializeException {
		return Element.of(QualityADEModule.NAMESPACE_URI, "ComponentList");
	}

	@Override
	public void writeChildElements(ComponentList object, Namespaces namespaces, XMLWriter writer)
			throws ObjectSerializeException, XMLWriteException {
		for (PolygonIdListProperty listProperty : object.getComponents()) {
			writer.writeElementUsingSerializer(Element.of(QualityADEModule.NAMESPACE_URI, "component"), listProperty,
					PolygonIdListPropertyAdapter.class, namespaces);
		}
	}
}
